import sys
import dill
import os
import pandas

from collections import OrderedDict

from revoscalepy import *
from revoscalepy.RxSerializable import rx_native_call
from revoscalepy.utils.RxTrace import rx_runtime_trace, RxTraceLevel


def exec_by_fun_mapper(args, _native_call_function):
    work_dir = args[1]
    job_py_data_path = args[2]
    num_tasks = int(args[3])

    rx_runtime_trace("start", RxTraceLevel.INFO)

    try:
        with open(job_py_data_path, "rb") as job_py_data_file:
            job_py_data = dill.load(job_py_data_file)

        function = job_py_data['func']

        for task_id in range(0, num_tasks):
            rx_runtime_trace("process execby mapper task with id {0}".format(task_id), RxTraceLevel.DEBUG)

            # build input data source
            xdf_name = "PXDF" + str(task_id)
            in_data_path = os.path.join(work_dir, xdf_name)
            in_data = RxXdfData(file=in_data_path)

            # deserialize binary key file and get key data.frame
            key_path = os.path.join(work_dir, xdf_name, "metadata", xdf_name + ".xdfk")

            rx_runtime_trace("read key from {0}".format(key_path), RxTraceLevel.DEBUG)

            call_params = {
                'USE': 'DeserializeSparkDataSet',
                'inputObjectPath': key_path
            }
            key_data_frame = _native_call_function(functionname='Util', params=call_params)

            # convert keys from pandas data frame to dict
            keys = OrderedDict()
            for key_name in key_data_frame.columns.tolist():
                keys[key_name] = key_data_frame[key_name][0]

            # invoke UDF
            result = None

            function_args = {
                'keys': keys.copy(),
                'data': in_data
            }

            if isinstance(job_py_data['funcParams'], dict):
                function_args.update(job_py_data['funcParams'])

            try:
                rx_runtime_trace("call udf start", RxTraceLevel.INFO)
                result = function(**function_args)
                rx_runtime_trace("call udf finish", RxTraceLevel.INFO)

                status = "OK"
            except Exception as e:
                status = e

            # construct out_data
            out_data_dict = OrderedDict()
            out_data_dict.update(keys)
            out_data_dict['result'] = [result]
            out_data_dict['status'] = [status]
            out_data_df = pandas.DataFrame(out_data_dict)

            # save out_data
            with open(os.path.join(work_dir, "iroFile{0}.iro".format(str(task_id))), "wb") as out_data_file:
                dill.dump(out_data_df, out_data_file)
    except Exception as e:
        sys.stderr.write("{0}\n".format(str(e)))
        sys.exit(1)
    finally:
        rx_runtime_trace("finish", RxTraceLevel.INFO)


if __name__ == '__main__':
    exec_by_fun_mapper(sys.argv, rx_native_call)
