import sys
import dill
import os


def exec_fun_mapper(args):
    work_dir = args[1]
    job_py_data_path = args[2]
    num_tasks = int(args[3])
    task_chunk_size = int(args[4])
    task_partition = int(args[5])

    try:
        with open(job_py_data_path, "rb") as job_py_data_file:
            job_py_data = dill.load(job_py_data_file)

        function = job_py_data['func']
        function_args_tasks = job_py_data['funcParams']

        local_state = job_py_data['localState']
        if isinstance(local_state, dict):
            locals().update(local_state)

        continue_on_failure = job_py_data['continueOnFailure']

        task_begin_id = task_chunk_size * task_partition
        task_end_id = min(num_tasks, task_chunk_size * (task_partition + 1))

        for task_id in range(task_begin_id, task_end_id):
            function_args = function_args_tasks[task_id]

            # invoke UDF
            try:
                result = function(**function_args)
            except Exception as e:
                if not continue_on_failure:
                    raise e
                result = e

            # save out_data
            with open(os.path.join(work_dir, "rxElem{0}.pyOutData".format(str(task_id))), "wb") as out_data_file:
                dill.dump(result, out_data_file)
    except Exception as e:
        sys.stderr.write("{0}\n".format(str(e)))
        sys.exit(1)


if __name__ == '__main__':
    exec_fun_mapper(sys.argv)
