import sys
import dill
import os
import subprocess
from revoscalepy.RxSerializable import rx_native_call
from revoscalepy.computecontext.RxSpark import load_transform_function
from revoscalepy.utils.RxTrace import RxTraceLevel
from revoscalepy.utils.RxTrace import rx_runtime_trace


def mapper(args, _native_call_function, os_listdir, subprocess_call):

    input_object_path = args[1]
    iro_output_path_base = args[2]
    local_final_iro_path = args[3]
    input_data_path_base = args[4]
    output_data_in_hdfs = (args[5].lower() == 'true')
    output_data_path_base = args[6]
    trace_level = int(args[7])
    process_type = 'WORKER'
    log_dir = args[8]
    transform_func = None

    rx_runtime_trace("start", RxTraceLevel.INFO)

    try:
        if trace_level > 0:
            call_params = {
                              'USE': 'InitializeLogging',
                              'traceLevel': trace_level,
                              'processType': process_type,
                              'logDir': log_dir
                          }
            _native_call_function(functionname='Util', params=call_params)

        xdfd_files = os_listdir(input_data_path_base + '/data')
        iro_paths = list()

        work_dir = os.path.abspath(os.path.join(input_object_path, os.pardir))
        transform_func, transform_objs = load_transform_function(work_dir)
       
        for xdfd_file in xdfd_files:
            input_data_path = os.path.join(input_data_path_base, 'data', xdfd_file)
            task_id = int(xdfd_file.split('_')[-1].split('.')[0]) - 1
            task_attempt_id = 0
            output_data_load_path = output_data_path_base + str(task_id)
            call_params = {
                              'USE': 'ComputeHadoopTask',
                              'taskId': task_id,
                              'taskAttemptId': task_attempt_id,
                              'inputObjectPath': input_object_path,
                              'inputInHdfs': False,
                              'outputObjectPathBase': iro_output_path_base,
                              'outputInHdfs': False,
                              'inputDataInHdfs': False,
                              'inputDataPath': input_data_path,
                              'transformObjects': transform_objs
                          }
            if not output_data_in_hdfs: 
                call_params['outputDataPathBase'] = output_data_path_base
                call_params['outputDataInHdfs'] = False

            rx_runtime_trace("compute task, " + str(xdfd_file), RxTraceLevel.INFO)
            _native_call_function(functionname='Util', params=call_params, rxTransformFunction=transform_func)
            iro_paths.append(iro_output_path_base + str(task_id) + '.iro')

            index = 0
            for out_xdfd_file in os_listdir(output_data_path_base + '/data'):
                output_xdfd_file = output_data_path_base + '/data/' + out_xdfd_file  
                subprocess_call(['mv', output_xdfd_file, output_data_load_path + '_' + str(index)])
                index = index + 1

        rx_runtime_trace("update iro", RxTraceLevel.INFO)
        call_params = {
                          'USE': 'UpdateIRO',
                          'fullPath': iro_paths,
                          'inHdfs': False
                      }
        _native_call_function(functionname='Util', params=call_params)

        rx_runtime_trace("write iro", RxTraceLevel.INFO)
        call_params = {
                          'USE': 'WriteIRO',
                          'outputObjectPath': local_final_iro_path,
                          'outputInHdfs': False
                      }
        _native_call_function(functionname='Util', params=call_params)
    except Exception as e:
        print(e)
        sys.exit(1)

    rx_runtime_trace("finish", RxTraceLevel.INFO)


if __name__ == '__main__':
    mapper(sys.argv, rx_native_call, os.listdir, subprocess.call)
