import sys
import dill
import os
from revoscalepy.RxSerializable import rx_native_call
from revoscalepy.computecontext.RxSpark import load_transform_function
from revoscalepy.utils.RxTrace import RxTraceLevel
from revoscalepy.utils.RxTrace import rx_runtime_trace


def text_to_xdf_mapper(args, _native_call_function):

    input_text_file_path = args[1]
    output_xdf_root_path = args[2]
    task_id = int(args[3])
    input_object_path = args[4]
    iro_output_path = args[5]
    trace_level = int(args[6])
    process_type = "WORKER"
    log_dir = args[7]
    transform_func = None

    rx_runtime_trace("start", RxTraceLevel.INFO)

    try:
        if trace_level > 0:
            call_params = {
                              'USE': 'InitializeLogging',
                              'traceLevel': trace_level,
                              'processType': process_type,
                              'logDir': log_dir
                          }
            _native_call_function(functionname='Util', params=call_params)

        rx_runtime_trace("set split info", RxTraceLevel.INFO)
        call_params = {
                          'USE': 'SetSplitInfo',
                          'fileName': input_text_file_path,
                          'start': 0,
                          'length': -1
                      }
        _native_call_function(functionname='Util', params=call_params)

        rx_runtime_trace("compute task", RxTraceLevel.INFO)

        work_dir = os.path.abspath(os.path.join(input_object_path, os.pardir))
        transform_func, transform_objs = load_transform_function(work_dir)

        task_attempt_id = 0
        call_params = {
                          'USE': 'ComputeHadoopTask',
                          'taskId': task_id,
                          'taskAttemptId': task_attempt_id,
                          'inputObjectPath': input_object_path,
                          'inputInHdfs': False,
                          'outputObjectPathBase': iro_output_path,
                          'outputInHdfs': False,
                          'inputDataInHdfs': False,
                          'outputDataPathBase': output_xdf_root_path,
                          'outputDataInHdfs': False,
                          'transformObjects': transform_objs
                      }

        _native_call_function(functionname='Util', params=call_params, rxTransformFunction=transform_func)
 
    except Exception as e:
        print(e)
        sys.exit(1)

    rx_runtime_trace("finish", RxTraceLevel.INFO)


if __name__ == "__main__":
    text_to_xdf_mapper(sys.argv, rx_native_call)
