library(RevoScaleR)
local(rxOptions())

# For new Spark prototype (2015-11-25)
args <- commandArgs(trailingOnly = TRUE)

'Mapper' <- function(args, rxNativeCall) {

    inputObjectPath       <- args[1]
    iroOutputPathBase     <- args[2]
    localFinalIroPath     <- args[3]
    inputDataPathBase     <- args[4]
    outputDataInHdfs      <- as.logical(args[5])
    outputDataPathBase    <- args[6]
    outputDataName        <- basename(outputDataPathBase)
    traceLevel            <- as.integer(args[7])
    processType           <- "WORKER"
    logDir                <- args[8]


    #trace(paste(args, collapse=" "))
    if(traceLevel>0) {
        rxNativeCall('RxUtil', list(USE='InitializeLogging', traceLevel = traceLevel, processType = processType, logDir = logDir))
    }

    tryCatch({
        xdfdFiles <- list.files(file.path(inputDataPathBase, "data"))

        iroPaths <- unlist("")
        iroPathsIndex <- 1
        for (xdfdFile in xdfdFiles) {
            inputDataPath <- paste(inputDataPathBase,"data",xdfdFile, sep="/")

            taskId <- as.integer(gsub("[^0-9]", "", tail(unlist(strsplit(xdfdFile,'_')),n=1) )) - 1
            taskAttemptId <- 0

            outputDataLoadPath <- paste0(outputDataPathBase,taskId)

            if (outputDataInHdfs) {
                rxUtilParams <- list(USE='ComputeHadoopTask', 
                    taskId=taskId, 
                    taskAttemptId=taskAttemptId, 
                    inputObjectPath=inputObjectPath, 
                    inputInHdfs = FALSE, 
                    outputObjectPathBase=iroOutputPathBase, 
                    outputInHdfs = FALSE,
                    inputDataInHdfs = FALSE,
                    inputDataPath = inputDataPath)
            } else {
                rxUtilParams <- list(USE='ComputeHadoopTask', 
                    taskId=taskId, 
                    taskAttemptId=taskAttemptId, 
                    inputObjectPath=inputObjectPath, 
                    inputInHdfs = FALSE, 
                    outputObjectPathBase=iroOutputPathBase, 
                    outputInHdfs = FALSE,
                    inputDataInHdfs = FALSE,
                    inputDataPath = inputDataPath,
                    outputDataPathBase = outputDataPathBase,
                    outputDataInHdfs = FALSE)
            }

             rxNativeCall( 'RxUtil', rxUtilParams)
             iroPaths[iroPathsIndex] <- paste(iroOutputPathBase, taskId, ".iro", sep="")
             iroPathsIndex <- iroPathsIndex + 1

            index <- 0 
            for ( outXdfdFile in list.files(file.path(outputDataPathBase, "data"))) {
                outputXdfdFile <- paste(outputDataPathBase,"data", outXdfdFile, sep="/")
                system(paste("mv", outputXdfdFile, paste0(outputDataLoadPath, '_', index)))
                index <- index + 1
            }
        } 
    

        # do local reduce in mapper worker
        rxNativeCall('RxUtil', list(USE='UpdateIRO', fullPath=iroPaths, inHdfs = FALSE))
        #trace("updating IROs from ", paste(iroPaths, collapse=","))

        rxNativeCall('RxUtil', list(USE='WriteIRO', outputObjectPath=localFinalIroPath, outputInHdfs = FALSE))
        #trace("writing IRO to ", localFinalIroPath)
    }, error = function(e) {
        print(e)
        quit("no", status = 1)
    })

}

if(length(args)>0) {
    if(length(args)==8) {
        Mapper(args, .Call)
    } else {
        stop(paste0("Internal Error: Mapper.R script does not get 8 arguments as expected. ",
                    "It actually gets ", length(args), " arguments.")) 

    }
}
