library(RevoScaleR)

# For new Spark prototype (2015-11-25)
args <- commandArgs(trailingOnly = TRUE)


"SingleMapper" <- function(args, rxNativeCall) {

    inputDataPath        <- args[1]
    inputDataInHdfs      <- as.logical(args[2])
    partitionIndex       <- as.numeric(args[3])
    inputObjectPath      <- args[4]
    iroOutputPath        <- args[5]
    outputDataInHdfs     <- as.logical(args[6])
    outputDataPathBase   <- args[7]
    traceLevel            <- as.integer(args[8])
    processType           <- "WORKER"
    logDir                <- args[9]

    if(traceLevel>0) {
        rxNativeCall('RxUtil', list(USE='InitializeLogging', traceLevel = traceLevel, processType = processType, logDir = logDir))
    }


    rxNativeCall( 'RxUtil', list( USE='SetSplitInfo', fileName=inputDataPath, start=0, length=-1 ) ) # length of -1 means read all bytes

    taskId <- as.integer(partitionIndex)
    taskAttemptId <- 0
    if (outputDataInHdfs) {
        rxNativeCall( 'RxUtil', list(USE='ComputeHadoopTask', 
                                     taskId=taskId, 
                                     taskAttemptId=taskAttemptId, 
                                     inputObjectPath=inputObjectPath, 
                                     inputInHdfs = FALSE, 
                                     outputObjectPathBase=iroOutputPath, 
                                     outputInHdfs = FALSE,
                                     inputDataInHdfs = inputDataInHdfs)
        )
    } else {
        rxNativeCall( 'RxUtil', list(USE='ComputeHadoopTask', 
                                     taskId=taskId, 
                                     taskAttemptId=taskAttemptId, 
                                     inputObjectPath=inputObjectPath, 
                                     inputInHdfs = FALSE, 
                                     outputObjectPathBase=iroOutputPath, 
                                     outputInHdfs = FALSE,
                                     inputDataInHdfs = inputDataInHdfs,
                                     outputDataPathBase = outputDataPathBase,
                                     outputDataInHdfs = FALSE)
        )
    }
}


if(length(args)>0) {
    if(length(args)==9) {
        SingleMapper(args, .Call)
    } else {
        stop(paste0("Internal Error: SingleMapper.R script does not get 9 arguments as expected. ",
                    "It actually gets ", length(args), " arguments."))
    }
}

