library(RevoScaleR)

# For new Spark prototype (2015-11-25)
args <- commandArgs(trailingOnly = TRUE)


"TextToXdfMapper" <- function(args, rxNativeCall){

    inputTextFilePath <- args[1]
    outputXdfRootPath <- args[2]
    partitionIndex <- as.numeric(args[3])
    inputObjectPath <- args[4]
    iroOutputPath <- args[5]

    traceLevel            <- as.integer(args[6])
    processType           <- "WORKER"
    logDir                <- args[7]

    if(traceLevel>0){
        rxNativeCall('RxUtil', list(USE='InitializeLogging', traceLevel = traceLevel, processType = processType, logDir = logDir))
    }

    rxNativeCall( 'RxUtil', list( USE='SetSplitInfo', fileName=inputTextFilePath, start=0, length=-1 ) ) # length of -1 means read all bytes

    taskId <- as.integer(partitionIndex)
    taskAttemptId <- 0
    rxNativeCall( 'RxUtil', list(USE='ComputeHadoopTask', 
                          taskId=taskId, 
                          taskAttemptId=taskAttemptId, 
                          inputObjectPath=inputObjectPath, 
                          inputInHdfs = FALSE, 
                          outputObjectPathBase=iroOutputPath, 
                          outputInHdfs = FALSE,
                          inputDataInHdfs = FALSE,
                          outputDataPathBase = outputXdfRootPath,
                          outputDataInHdfs = FALSE)
    )

}


if(length(args)>0) {
    if(length(args)==7) {
        TextToXdfMapper(args, .Call)
    } else {
        stop(paste0("Internal Error: TextToXdfMapper.R script does not get 7 arguments as expected. ",
                    "It actually gets ", length(args), " arguments."))
    }
}
