#!/usr/bin/env python

import shlex, subprocess
import os
import sys
import fnmatch

def runUnix(command, halt_on_error=True, removeFinalNewline=False):
    child = os.popen(command)
    data = child.read()
    err = child.close()
    if err and halt_on_error:
        raise RuntimeError(command + ' failed w/ exit code ' + err)
    if removeFinalNewline and data.endswith("\n"):
        data = data.rstrip("\n")
    return data

def getJarPath():
   # if we have a command-line argument, use that as a path to look for the jar file
   if (len(sys.argv) > 1):
      temp_path = sys.argv[1]
      # check to see if the path is valid
      if (os.path.exists(os.path.abspath(temp_path))):
         jarPath = os.path.abspath(temp_path)
      else:
         print('ERROR: \'' + temp_path + '\' Does not appear to be a valid path')
         sys.exit(1)
   else:
      jarPath = os.path.dirname(os.path.realpath(__file__))
   return jarPath


def hadoopGetEnv(envvar, notassigned = '') :
    jarPath = getJarPath()
    cmd = "hadoop jar " + jarPath + "/jar/HadoopEnv-1.0-SNAPSHOT.jar com.revolutionanalytics.hadooputils.HadoopEnv " + envvar + " 2>/dev/null"
    try:
        res = runUnix(cmd,removeFinalNewline=True)
    except:
        return ""
    if res.endswith("not assigned.") :
        res = notassigned
    else :
        res = res.lstrip(envvar + "=")
    if res.endswith("..") :
        res = os.path.dirname(os.path.dirname(res))
    return res

def find(name, path) :
    for root, dirs, files in os.walk(path):
        if name in files:
            return os.path.join(root, name)
            
def findInHadoopDirs(pattern, path) :
    hadoopTop = os.path.dirname(path)
    hadoopPaths = []
    result = []
    if not os.path.isdir(hadoopTop):
        return ""
    for files in os.listdir(hadoopTop) :
        newPath=os.path.join(hadoopTop, files)
        if os.path.isdir(newPath) and files.startswith("hadoo") and not files.endswith("httpfs") and not files.endswith("hdfs") and not files.endswith("yarn") :
            hadoopPaths.append(newPath)  
    if hadoopTop.endswith("lib") and os.path.exists(os.path.join(os.path.dirname(hadoopTop), "lib64")) :
        hadoopPaths.append(os.path.join(os.path.dirname(hadoopTop), "lib64"))
    for eachPath in hadoopPaths :
        for root, dirs, files in os.walk(eachPath):
            for name in files:
                if fnmatch.fnmatch(name, pattern):
                    result.append(os.path.join(root, name))
    return result                    
            
def findHadoopHome(path) :
    hadoopTop = os.path.dirname(path)
    hadoopCore = []
    hadoopCommon = []
    hadoopDevtools = []
    res = []
    if not os.path.isdir(hadoopTop):
        return ""
    for files in os.listdir(hadoopTop) :
        if os.path.isdir(os.path.join(hadoopTop, files)) and files.startswith("hadoo") and not files.endswith("httpfs") and not files.endswith("hdfs") and not files.endswith("yarn") :
            res.append(files)
    for eachpath in res :
        newPath = os.path.join(hadoopTop, eachpath)
        foundCore = find("hadoop-core.jar", newPath)
        if foundCore is not None :
            hadoopCore.append(foundCore)
        foundCommon = find("hadoop-common.jar", newPath)
        if foundCommon is not None :
            hadoopCommon.append(foundCommon)
        foundDevtools = find("hadoop-0.20-2-dev.tools.jar", newPath)
        if foundDevtools is not None :
            hadoopDevtools.append(foundDevtools) 
    if len(hadoopCore) :
        return os.path.dirname(hadoopCore.pop())
    elif len(hadoopCommon) :
        return os.path.dirname(hadoopCommon.pop())
    elif len(hadoopDevtools) :
        return os.path.dirname(hadoopDevtools.pop())
    else :
        return path    
       
 
def get_spark_class_path():
    has_pyspark = True
    try:
        from pyspark.sql import SparkSession
    except ImportError:
        has_pyspark = False
    
    spark_class_path = ""
    if has_pyspark:
        sparkSession = SparkSession.builder.getOrCreate()
        spark_class_path = ":" + sparkSession._jvm.java.lang.System.getProperty("java.class.path") 
    else: 
        os.system("echo print\(System.getProperty\(\\\"java.class.path\\\"\)\) > /tmp/mlserver-getVarScript")
        os.system("echo print\(System.exit\(0\)\) >> /tmp/mlserver-getVarScript")    
        try:
            spark_class_path = ":" + runUnix("spark-shell -i /tmp/mlserver-getVarScript 2>/dev/null | tail -n 1", removeFinalNewline=True)
        except:
            pass
        os.system("rm /tmp/mlserver-getVarScript")
 
    return spark_class_path 

    
final_path = []
hadoopClasspath = getJarPath()
fullHadoopClasspath = subprocess.Popen("export HADOOP_CLASSPATH=" + hadoopClasspath + ";hadoop classpath 2>/dev/null", stdout=subprocess.PIPE, shell=True).stdout.read()
fullHadoopClasspath = str(fullHadoopClasspath.decode())
# fullHadoopClasspath += ":" + hadoopClasspath
originalPath = fullHadoopClasspath

hadoopHome = hadoopGetEnv("HADOOP_HOME")


if hadoopHome == '' :
    hadoopHome = hadoopPrefix = hadoopGetEnv("HADOOP_PREFIX")
else :
    hadoopPrefix = hadoopHome

hadoopHome = findHadoopHome(hadoopHome)
has_hadoop = (hadoopHome != '')

if os.path.exists(hadoopHome + "/client-0.20/") :
    fullHadoopClasspath += ":" + hadoopHome + "/client-0.20/"
if os.path.exists(hadoopHome + "/client/") : 
    fullHadoopClasspath += ":" + hadoopHome + "/client/"
if os.path.exists("/usr/lib/hbase/lib") :
    fullHadoopClasspath += ":/usr/lib/hbase/lib/"

#dedup : in the classpath.
oldString = fullHadoopClasspath
while True:
	fullHadoopClasspath = oldString.replace( "::", ":" )
	if oldString is fullHadoopClasspath:
		break
	oldString = fullHadoopClasspath

# fix up malformed Pivotal HD classpath
fullHadoopClasspath = fullHadoopClasspath.replace( "/.//*", "/*" )

individualPaths = fullHadoopClasspath.split(":")

for indPath in individualPaths:
	curPath = os.path.abspath( indPath.rstrip().lstrip() )
	cleanIndPath = curPath.rstrip( "*" )
	if not os.path.exists(cleanIndPath):
		continue


	if cleanIndPath[-1] != "/":
		cleanIndPath = cleanIndPath + "/"

	# makes sure the directory itself is added to the final path	
	final_path.append( cleanIndPath)
	
	#print( curPath )
	#print( cleanIndPath )
	#print "-------------------------------"

	cmd = "ls " + curPath + " | grep -v \":\" | sed '/^$/d' | grep -i 'jar$\|xml$'"
	
	
	ret = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True).stdout.read()
	curListing = str(ret.decode()).split("\n")


	if len( curListing ) == 1 and curListing[0] == cleanIndPath:
		final_path.append( curListing )
	else:
		for curJar in curListing:
			if ( curJar.__len__() > 0 ):
				if ( curJar[0] == "/" ):
					#print( "case 1: ", curJar )
					final_path.append( curJar )
				else:
					#print( "case 2: ", cleanIndPath + curJar )
					final_path.append( str(cleanIndPath) + str(curJar) )


# we want the original path (without jar files), but also without stars


if has_hadoop is False:
    print("Hadoop is not found.")
    final_path.append(get_spark_class_path())
 
    

## Now we have our final classpath, our Hadoop Home, and we can get the Java Home; let's do that now
hadoopJavaHome = hadoopGetEnv("JAVA_HOME")
javaHome = os.environ.get("JAVA_HOME")
hadoopConfDir = hadoopGetEnv("HADOOP_CONF_DIR")


## Now let's get the location of the streaming jar file
hadoopStreaming = ""
cmd = 'which rpm'
child = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
rpm_cmd = str(child.communicate()[0]).split("\n")[0]
rc = child.returncode
if not rc :
    cmd = str(rpm_cmd) + ' -q mapr-mapreduce1 2>/dev/null'
    child = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True) 
    rpm_mapr = child.communicate()[0]
    rc = child.returncode
    if not rc :
        cmd = rpm_cmd + ' -ql ' + rpm_mapr + ' | grep -m 1 hadoop[^/]*streaming[^/]*jar$'
        child = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
        hadoopStreaming = str(child.communicate()[0]).split("\n")[0]
        rc = child.returncode
        if rc :
            hadoopStreaming = ""
else :
    cmd = 'which dpkg-query'
    child = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
    dpkg_query_cmd = str(child.communicate()[0]).split("\n")[0]
    rc = child.returncode
    if not rc :
        cmd = str(dpkg_query_cmd) + ' -W -f=\'${Package}\' mapr-mapreduce1 2>/dev/null'
        child = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
        dpkg_query_mapr = str(child.communicate()[0]).split("\n")[0]
        rc = child.returncode
        if not rc :
            cmd = dpkg_query_cmd + ' -L ' + dpkg_query_mapr + ' | grep -m 1 hadoop[^/]*streaming[^/]*jar$'
            child = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
            hadoopStreaming = str(child.communicate()[0]).split("\n")[0]
            rc = child.returncode
            if rc :
                hadoopStreaming = ""

if not len(hadoopStreaming) :
    hadoopStreamingLocs = findInHadoopDirs("hadoo*stream*.jar", hadoopPrefix)
    if len(hadoopStreamingLocs) :
        for path in hadoopStreaming :
            if not os.path.islink(path) :
                hadoopStreaming = path
                break 
        if not len(hadoopStreaming) :
            hadoopStreaming=hadoopStreamingLocs.pop()

if not len(hadoopStreaming):
    print("Warning: Could not find hadoop streaming jar")


file = open("RevoHadoopEnvVars.site", "w")

if len(final_path) > 0:
    file.write("export CLASSPATH=" + ":".join(final_path) + '\n')

if hadoopHome is not "":
    file.write("export HADOOP_HOME=" + hadoopHome + "\n")
    file.write("export HADOOP_CMD=hadoop\n")
# 1. LIBHDFS_OPTS is used by libhdfs.so as arguments for creating JVM.
#    Please reference: 
#    https://msdata.visualstudio.com/Revolution R/_git/External-libhdfs?path=%2Fsrc%2Fjni_helper.c
# 2. Since linux kernel update on 06/22/2017, need to increase stack size for JVM hosted by Bxlserver.
#    https://bugs.launchpad.net/ubuntu/+source/linux/+bug/1699772
# 3. Add -Xrs in LIBHDFS_OPTS. It will avoid signals (e.g. SIGINT from Ctrl-C) from R console affecting JVM.
#    More details about -Xrs: http://www.oracle.com/technetwork/java/javase/signals-139944.html
if os.path.exists(hadoopHome + "/lib/native/") :
    file.write("export LIBHDFS_OPTS=" + "-Djava.library.path=" + hadoopHome + "/lib/native/" + " -Xrs -Xss4m\n")
else:
    file.write("export LIBHDFS_OPTS=-Xrs -Xss4m\n")

# if hadoopJavaHome != javaHome, we want to use the Hadoop one...
if hadoopJavaHome != javaHome and hadoopJavaHome is not "" :
    file.write("export JAVA_HOME=" + hadoopJavaHome + "\n")

if hadoopPrefix is not "":
    file.write("export REVOHADOOPPREFIX=" + hadoopPrefix + "\n")

if len(hadoopConfDir) :
    if len(hadoopStreaming) :
        file.write("export REVOHADOOPSWITCHES=-libjars=" + hadoopConfDir + "," + hadoopStreaming + "\n")
    else :
        file.write("export REVOHADOOPSWITCHES=-libjars=" + hadoopConfDir + "\n")
else :
    file.write("export REVOHADOOPSWITCHES=\"\"\n")
    
# Not sure yet if this going to work from here
if len(hadoopStreaming) :
    file.write("export HADOOP_STREAMING=" + hadoopStreaming + "\n")

file.close()

print("Wrote RevoHadoopEnvVars.site to " + os.getcwd() + ".")

