#!/bin/bash

# --- defaults ---------------------------------------------------------
# load from env-final.json
MRS_HADOOP_PATH="%mrsHadoopPath%"
INTERPRETER_PATH="%interpreterPath%"
MASTER_INTERPRETER_PATH="%masterInterpreterPath%"
SHARE_DIR="%shareDir%"
USER="%user%"
SESSION_PID="%sessionPid%"
APP_ID="%appId%"
WAIT="%wait%"
NAME_NODE="%nameNode%"
PORT="%port%"
IDLE_TIMEOUT="%idleTimeout%"
EXECUTOR_CORES="%executorCores%"
EXECUTOR_MEM="%executorMem%"
DRIVER_MEM="%driverMem%"
OVERHEAD_MEM="%overheadMem%"
NUM_EXECUTORS="%numExecutors%"
EXTRA_CONFIG="%extraConfig%"
LOGGING_LEVEL="%loggingLevel%"
PERSISTENT_RUN="%persistentRun%"
TMP_FS_WORK_DIR="%tmpFSWorkDir%"
SERVER_TYPE="%serverType%"
MASTER="%master%"
# get from command
JOB_GUID="%jobGUID%"

# --- command-line options ---------------------------------------------
while [ "$1" != "" ]; do
  case "$1" in
    --mrsHadoopPath )   shift
                        MRS_HADOOP_PATH=$1
                        ;;
    --interpreterPath ) shift
                        INTERPRETER_PATH=$1
                        ;;
    --masterInterpreterPath ) shift
                        MASTER_INTERPRETER_PATH=$1
                        ;;
    --shareDir )        shift
                        SHARE_DIR=$1
                        ;;
    --user )            shift
                        USER=$1
                        ;;
    --SessionPid )      shift
                        SESSION_PID=$1
                        ;;
    --appId )           shift
                        APP_ID=$1
                        ;;
    --wait )            shift
                        WAIT=$1
                        ;;
    --nameNode )        shift
                        NAME_NODE=$1
                        ;;
    --port )            shift
                        PORT=$1
                        ;;
    --idleTimeout )     shift
                        IDLE_TIMEOUT=$1
                        ;;
    --executorCores )   shift
                        EXECUTOR_CORES=$1
                        ;;
    --executorMem )     shift
                        EXECUTOR_MEM=$1
                        ;;
    --driverMem )       shift
                        DRIVER_MEM=$1
                        ;;
    --overheadMem )     shift
                        OVERHEAD_MEM=$1
                        ;;
    --numExecutors )    shift
                        NUM_EXECUTORS=$1
                        ;;
    --extraConfig )     shift
                        EXTRA_CONFIG="$1"
                        ;;
    --loggingLevel )    shift
                        LOGGING_LEVEL=$1
                        ;;
    --jobGUID )         shift
                        JOB_GUID=$1
                        ;;
    --persistentRun )   shift
                        PERSISTENT_RUN=$1
                        ;;
    --tmpFSWorkDir )    shift
                        TMP_FS_WORK_DIR=$1
                        ;;
    --serverType )      shift
                        SERVER_TYPE=$1
                        ;;
    --master )          shift
                        MASTER=$1
                        ;;
    * )                 ;;
  esac
  shift
done

HADOOP_FOLDER_PATH=$(dirname "$(readlink -nf ${MRS_HADOOP_PATH})")
SUBMIT_LAUNCHER_PATH=${HADOOP_FOLDER_PATH}/workflow/common/RevoSparkSubmitLauncher
# --- script logic -----------------------------------------------------

# load environment file
SITE_PATH="${HADOOP_FOLDER_PATH}"/RevoHadoopEnvVars.site

if [ ! -e "${SITE_PATH}" ] ; then
  SITE_PATH="${HOME}/.RevoHadoopEnvVars.site"
fi
if [ -e "${SITE_PATH}" ] ; then
  while read LINE ; do
    LINE="${LINE/export /}"
    VAR_NAME=${LINE%%=*}
    VAR_VALUE=$(eval "echo \${${VAR_NAME}}")
    if [ -z "${VAR_VALUE}" ] ; then
      export ${LINE}
    fi
  done < "${SITE_PATH}"
else
  echo "cannot find RevoHadoopEnvVars.site, aborting" >&2
  exit 1
fi

FIFO_TO_SPARK=${SHARE_DIR}/RevoSparkFifoToSpark-${USER}-${SESSION_PID}-${APP_ID}
FIFO_FROM_SPARK=${SHARE_DIR}/RevoSparkFifoFromSpark-${USER}-${SESSION_PID}-${APP_ID}

"${SUBMIT_LAUNCHER_PATH}" "$FIFO_TO_SPARK" "$FIFO_FROM_SPARK" "${INTERPRETER_PATH}" \
                                       "${NAME_NODE}" "${PORT}" "${USER}" \
                                       "${SESSION_PID}" "${APP_ID}" "${JOB_GUID}" \
                                       "${WAIT}" "${IDLE_TIMEOUT}" \
                                       "${EXECUTOR_CORES}" "${EXECUTOR_MEM}" \
                                       "${DRIVER_MEM}" "${OVERHEAD_MEM}" \
                                       "${NUM_EXECUTORS}" "`echo ${EXTRA_CONFIG} | sed -e 's/%DOUBLEQUOTE%/\"/g'`" \
                                       "${LOGGING_LEVEL}" "${TMP_FS_WORK_DIR}" \
                                       "${SERVER_TYPE}" "${MASTER}" "${MASTER_INTERPRETER_PATH}"

if [ $? -ne 0 ]; then
    echo "Error while running RevoSparkSubmitLauncher" >&2
    exit 1
fi
