#!/bin/bash

# --- defaults ---------------------------------------------------------
CLUSTER_GUID_DIR="%clusterGUIDDir%"


# --- command-line options ---------------------------------------------
while [ "$1" != "" ]; do
  case $1 in
    --clusterGUIDDir )  shift
                        CLUSTER_GUID_DIR=$1
                        ;;
    --msg-level )       shift
                        MSG_LEVEL=$1
                        ;;
    --max-line )        shift
                        MAX_LINE=$1
                        ;;
    * )                 ;;
  esac
  shift
done

if [ -z $MSG_LEVEL ] || [[ " ERROR INFO " != *" ${MSG_LEVEL} "* ]]; then
  echo "Invalid msg-level value. It should be ERROR/INFO." >&2
  exit 1
fi

if [ ${MSG_LEVEL} == "ERROR" ]; then
  LOG_FILE=jobLog.err
  OUTPUT_LINE=lastMonitorLine.err
else
  LOG_FILE=jobLog.info
  OUTPUT_LINE=lastMonitorLine.info
fi
OUTPUT_BLOCK=lastMonitorBlock.txt

if [ -z ${MAX_LINE} ]; then
  echo "max-line value is required." >&2
  exit 1
fi

if [ ${MAX_LINE} != "all" ] && [ ! "${MAX_LINE}" -eq "${MAX_LINE}" ]; then
  echo "Invalid max-line value. It should be number like 128 or 'all'." >&2
  exit 1
fi

# --- script logic -----------------------------------------------------

MYSTART=1
if [ -f ${CLUSTER_GUID_DIR}/${OUTPUT_LINE} ]
then
    MYSTART=$(cat ${CLUSTER_GUID_DIR}/${OUTPUT_LINE})
fi
if [ -f ${CLUSTER_GUID_DIR}/${LOG_FILE} ]
then
    NUMLINES=$(wc -l ${CLUSTER_GUID_DIR}/${LOG_FILE} | cut -f1 -d' ')
    let RESTLINES=$NUMLINES-$MYSTART+1
    if [ $NUMLINES -gt 0 ] && [ $RESTLINES -gt 0 ]
    then
        if [ "${MAX_LINE}" == "all" ]; then
            OUTLINES=$RESTLINES
        else
            if [ $RESTLINES -gt ${MAX_LINE} ]; then
                OUTLINES=${MAX_LINE}
            else
                OUTLINES=$RESTLINES
            fi
        fi
        tail -n +$MYSTART ${CLUSTER_GUID_DIR}/${LOG_FILE} | head -n $OUTLINES > ${CLUSTER_GUID_DIR}/${OUTPUT_BLOCK}
        let NEWCURLINE=$MYSTART+$OUTLINES
        echo $NEWCURLINE > ${CLUSTER_GUID_DIR}/${OUTPUT_LINE}
    else
        if [ -f ${CLUSTER_GUID_DIR}/${OUTPUT_BLOCK} ]
        then
            rm ${CLUSTER_GUID_DIR}/${OUTPUT_BLOCK}
        fi
        touch ${CLUSTER_GUID_DIR}/${OUTPUT_BLOCK}
    fi
else
    if [ -f ${CLUSTER_GUID_DIR}/${OUTPUT_BLOCK} ]
    then
        rm ${CLUSTER_GUID_DIR}/${OUTPUT_BLOCK}
    fi
    touch ${CLUSTER_GUID_DIR}/${OUTPUT_BLOCK}
fi


