#!/bin/bash

# --- defaults ---------------------------------------------------------
# load from env-final.json
REVO_UTILS_DIR="%revoUtilsDir%"
MRS_HADOOP_PATH="%mrsHadoopPath%"
INTERPRETER_PATH="%interpreterPath%"
MASTER_INTERPRETER_PATH="%masterInterpreterPath%"
SHARE_DIR="%shareDir%"
USER="%user%"
SESSION_PID="%sessionPid%"
APP_ID="%appId%"
WAIT="%wait%"
NAME_NODE="%nameNode%"
PORT="%port%"
IDLE_TIMEOUT="%idleTimeout%"
EXECUTOR_CORES="%executorCores%"
EXECUTOR_MEM="%executorMem%"
DRIVER_MEM="%driverMem%"
OVERHEAD_MEM="%overheadMem%"
NUM_EXECUTORS="%numExecutors%"
EXTRA_CONFIG="%extraConfig%"
LOGGING_LEVEL="%loggingLevel%"
PERSISTENT_RUN="%persistentRun%"
SKIP_LAUNCH_SPARK="%skipLaunchSpark%"
TMP_FS_WORK_DIR="%tmpFSWorkDir%"
SERVER_TYPE="%serverType%"
MASTER="%master%"
# get from command
CLUSTER_GUID_DIR="%clusterGUIDDir%"
JOB_GUID="%jobGUID%"

# --- command-line options ---------------------------------------------
while [ $# -gt 0 ]; do
  case "$1" in
    -revoUtilsDir)    REVO_UTILS_DIR="$2"; shift 2;;
    -mrsHadoopPath)   MRS_HADOOP_PATH="$2"; shift 2;;
    -interpreterPath) INTERPRETER_PATH="$2"; shift 2;;
    -masterInterpreterPath) MASTER_INTERPRETER_PATH="$2"; shift 2;;
    -shareDir)        SHARE_DIR="$2"; shift 2;;
    -user)            USER="$2"; shift 2;;
    -SessionPid)      SESSION_PID="$2"; shift 2;;
    -appId)           APP_ID="$2"; shift 2;;
    -wait)            WAIT="$2"; shift 2;;
    -nameNode)        NAME_NODE="$2"; shift 2;;
    -port)            PORT="$2"; shift 2;;
    -idleTimeout)     IDLE_TIMEOUT="$2"; shift 2;;
    -executorCores)   EXECUTOR_CORES="$2"; shift 2;;
    -executorMem)     EXECUTOR_MEM="$2"; shift 2;;
    -driverMem)       DRIVER_MEM="$2"; shift 2;;
    -overheadMem)     OVERHEAD_MEM="$2"; shift 2;;
    -numExecutors)    NUM_EXECUTORS="$2"; shift 2;;
    -extraConfig)     EXTRA_CONFIG="$2"; shift 2;;
    -loggingLevel)    LOGGING_LEVEL="$2"; shift 2;;
    -clusterGUIDDir)  CLUSTER_GUID_DIR="$2"; shift 2;;
    -jobGUID)         JOB_GUID="$2"; shift 2;;
    -persistentRun)   PERSISTENT_RUN="$2"; shift 2;;
    -skipLaunchSpark) SKIP_LAUNCH_SPARK="$2"; shift 2;;
    -tmpFSWorkDir)    TMP_FS_WORK_DIR="$2"; shift 2;;
    -serverType)      SERVER_TYPE="$2"; shift 2;;
    -master)          MASTER="$2"; shift 2;;
    *) echo "WARNING! unknown option $1"; shift;;
  esac
done

HADOOP_FOLDER_PATH=$(dirname "$(readlink -nf ${MRS_HADOOP_PATH})")
SAFE_GUARD_PATH=${HADOOP_FOLDER_PATH}/utils/SafeGuard.out
SUBMIT_LAUNCHER_PATH=${HADOOP_FOLDER_PATH}/workflow/common/RevoSparkSubmitLauncher
LAUNCHER_PATH=${HADOOP_FOLDER_PATH}/workflow/common/RevoSparkJobLauncher
KILLER_PATH=${HADOOP_FOLDER_PATH}/workflow/common/RevoSparkSubmitKiller
JSON_PATH=${CLUSTER_GUID_DIR}/job.json
# --- script logic -----------------------------------------------------

#Cannot use vanilla R here, since we need some options from Rprofile
if [ "R" == "${SERVER_TYPE}" ]; then
    #TODO: add telemetry for python
    ${SAFE_GUARD_PATH} ${MASTER_INTERPRETER_PATH} --slave -e 'library(RevoScaleR); RevoScaleR:::rxLogHadoopMRJob("${CLUSTER_GUID_DIR}", "RxSpark")' 2>&1 & > /dev/null
fi
touch ${CLUSTER_GUID_DIR}/jobLog.err
touch ${CLUSTER_GUID_DIR}/jobLog.info
touch ${CLUSTER_GUID_DIR}/lastMonitorBlock.txt
echo "uninitialized" > ${CLUSTER_GUID_DIR}/jobIdentifier.txt
MYUUID=$(cat /proc/sys/kernel/random/uuid)

# load environment file
SITE_PATH="${HADOOP_FOLDER_PATH}"/RevoHadoopEnvVars.site

if [ ! -e "${SITE_PATH}" ] ; then
  SITE_PATH="${HOME}/.RevoHadoopEnvVars.site"
fi
if [ -e "${SITE_PATH}" ] ; then
  while read LINE ; do
    LINE="${LINE/export /}"
    VAR_NAME=${LINE%%=*}
    VAR_VALUE=$(eval "echo \${${VAR_NAME}}")
    if [ -z "${VAR_VALUE}" ] ; then
      eval "export ${LINE}"
    fi
  done < "${SITE_PATH}"
else
  echo "failed" > ${CLUSTER_GUID_DIR}/$MYUUID
  mv ${CLUSTER_GUID_DIR}/$MYUUID ${CLUSTER_GUID_DIR}/jobStatus.txt
  echo "cannot find RevoHadoopEnvVars.site, aborting" >&2
  exit 1
fi

echo "queued" > ${CLUSTER_GUID_DIR}/$MYUUID
mv ${CLUSTER_GUID_DIR}/$MYUUID ${CLUSTER_GUID_DIR}/jobStatus.txt

FIFO_TO_SPARK=${SHARE_DIR}/RevoSparkFifoToSpark-${USER}-${SESSION_PID}-${APP_ID}
FIFO_FROM_SPARK=${SHARE_DIR}/RevoSparkFifoFromSpark-${USER}-${SESSION_PID}-${APP_ID}
# In non-blocking mode, multi jobs can run simultaneously in one R session, append job GUID to make each job unique.
if ! ${WAIT} ; then
  FIFO_TO_SPARK=${FIFO_TO_SPARK}-${JOB_GUID}
  FIFO_FROM_SPARK=${FIFO_FROM_SPARK}-${JOB_GUID}
fi
# Only clean Spark app associated with current R session in blocking mode.
# In non-blocking mode, we allow multi Spark apps within same R session.
if ${WAIT} ; then
  if ${PERSISTENT_RUN} ; then
    # persistent run mode will only clean Spark apps that are not submitted by current cc
    ${KILLER_PATH} ${USER} ${SESSION_PID} ${APP_ID} ${JOB_GUID} ${WAIT} clean session 
  else
    # non persistent run mode will kill all old Spark apps that are in current R session
    # Avoid old job receiving command from new job, since they share the same fifoToSpark
    ${KILLER_PATH} ${USER} ${SESSION_PID} ${APP_ID} ${JOB_GUID} ${WAIT} kill session
  fi
fi

if [ ! "${SKIP_LAUNCH_SPARK}" == "TRUE" ] && [ ! "${SKIP_LAUNCH_SPARK}" == "true" ]; then
    "${SUBMIT_LAUNCHER_PATH}" "$FIFO_TO_SPARK" "$FIFO_FROM_SPARK" "${INTERPRETER_PATH}" \
                                           "${NAME_NODE}" "${PORT}" "${USER}" \
                                           "${SESSION_PID}" "${APP_ID}" "${JOB_GUID}" \
                                           "${WAIT}" "${IDLE_TIMEOUT}" \
                                           "${EXECUTOR_CORES}" "${EXECUTOR_MEM}" \
                                           "${DRIVER_MEM}" "${OVERHEAD_MEM}" \
                                           "${NUM_EXECUTORS}" "`echo ${EXTRA_CONFIG} | sed -e 's/%DOUBLEQUOTE%/\"/g'`" \
                                           "${LOGGING_LEVEL}" "${TMP_FS_WORK_DIR}" \
                                           "${SERVER_TYPE}" "${MASTER}" "${MASTER_INTERPRETER_PATH}"

    if [ $? -ne 0 ]; then
        echo "Error while running RevoSparkSubmitLauncher" >&2
        exit 1
    fi
fi

(${LAUNCHER_PATH} \
 ${FIFO_TO_SPARK} \
 ${FIFO_FROM_SPARK} \
 ${JSON_PATH} 1>${CLUSTER_GUID_DIR}/jobLog.info  2>${CLUSTER_GUID_DIR}/jobLog.err;
 # Send stop to spark if we're in non-persist mode
 if ! ${PERSISTENT_RUN} ; then
     ${KILLER_PATH} ${USER} ${SESSION_PID} ${APP_ID} ${JOB_GUID} ${WAIT} stop ${SHARE_DIR}/RevoSparkFifoToSpark-${USER}-${SESSION_PID}-${APP_ID}-${JOB_GUID}>/dev/null 2>&1
 fi)>/dev/null 2>&1 &
COMMANDPID=$!
echo ${COMMANDPID} > ${CLUSTER_GUID_DIR}/jobIdentifier.txt
echo ${COMMANDPID}
