import sys
import json
import os
import warnings

def check_and_assign_env_api_values(input_api, check_api):
    if "pySessionPid" in input_api:
        input_api["sessionPid"] = input_api["pySessionPid"]
        input_api.pop("pySessionPid", None)
    if "hadoopPyPath" in input_api:
        input_api["interpreterPath"] = input_api["hadoopPyPath"]
        input_api.pop("hadoopPyPath", None)
        
    for key in check_api.keys():
        if key in input_api:
            check_api[key] = input_api[key]
        else:
            if check_api[key].strip() == "":
                raise Exception("DATA PROTOCOL ERROR: Server is missing input data (%s) from client. " % key + \
                                "Client and server RevoScaleR package version probably does not match.")
            else:
                warnings.warn("DATA PROTOCOL WARNING: Server is missing input data (%s) from client. " % key + \
                              "Use server side default value. " + \
                              "Client and server RevoScaleR package version probably does not match.")

    for key in input_api.keys():
        if key in ["envVersion"]:
            continue
        if key not in check_api:
            warnings.warn("DATA PROTOCOL WARNING: Server received unknown input data (%s) from client. " % key + \
                          "Ignore the data. " + \
                          "Client and server RevoScaleR package version probably does not match.")
    # assign server type
    check_api["serverType"] = "Python"
    return check_api

def validate_job_file(job):
    if "jobType" not in job:
        raise Exception("DATA PROTOCOL ERROR: Invalid format of job.json. Missing slot of jobType.")
    job_type = job["jobType"]

    # Currently we don't have plan to migrate hadoop to new workflow
    # Just keep it.
    if job_type not in ["spark-hpc", "spark-hpa", "hadoop-hpc", "hadoop-hpa"]:
        raise Exception("DATA PROTOCOL ERROR: Invalid value of jobType in job.json. " + \
                        "Accepted value: spark-hpc, spark-hpa, hadoop-hpc, hadoop-hpa")

    if "function" not in job:
        raise Exception("DATA PROTOCOL ERROR: Missing 'function' value in job.json file. " + \
                        "It is required for starting Spark job.")

    if "jobGUID" not in job:
        raise Exception("DATA PROTOCOL ERROR: Missing 'jobGUID' value in job.json file. " + \
                        "It is required for starting Spark job.")


def main():
    work_dir = sys.argv[1]
    src_dir = sys.argv[2]

    with open(os.path.join(work_dir, "env.json")) as env_json_file:
        env = json.load(env_json_file)
    with open(os.path.join(work_dir, "job.json")) as job_json_file:
        job = json.load(job_json_file)
    with open(os.path.join(src_dir, "..", "templates", "env.json")) as env_api_template_file:
        env_api = json.load(env_api_template_file)

    validate_job_file(job)
    env_checked_api = check_and_assign_env_api_values(env, env_api)

    with open(os.path.join(work_dir, "env-final.json"), 'w') as outfile:
        json.dump(env_checked_api, outfile, indent=4, sort_keys=True)

if __name__ == "__main__":
    main()
