import sys
import json
import os

def fill_template(template_file_name, out_file_name, env, job, src_dir, work_dir):
    with open(os.path.join(src_dir, "..", "templates", template_file_name), 'r') as template_file:
        template=template_file.read()

    template = do_template_fill(template, env, job, work_dir)
    with open(os.path.join(work_dir, out_file_name), "w") as out_file:
        out_file.write(template)


def do_template_fill(template, env, job, work_dir):
    template = fill_env(template, env)
    template = template.replace("%jobGUID%", job["jobGUID"])
    template = template.replace("%clusterGUIDDir%", work_dir)
    return(template)


def fill_env(template, env):
    for key in env.keys():
        if key == "extraConfig":
            template = template.replace("%" + key + "%", str(env[key]).replace("\"", "%DOUBLEQUOTE%"))
        else:
            template = template.replace("%" + key + "%", str(env[key]))
    return(template)


def main():
    work_dir = sys.argv[1]
    src_dir = sys.argv[2]
    command = sys.argv[3]

    with open(os.path.join(work_dir, "env-final.json")) as env_json_file:
        env = json.load(env_json_file)
    with open(os.path.join(work_dir, "job.json")) as job_json_file:
        job = json.load(job_json_file)

    if command == "connect":
        fill_template("spark-connect-template.sh", "spark-connect.sh", env, job, src_dir, work_dir)
    if command == "disconnect":
        fill_template("spark-disconnect-template.sh", "spark-disconnect.sh", env, job, src_dir, work_dir)
    if command == "applist":
        fill_template("spark-list-template.sh", "spark-list.sh", env, job, src_dir, work_dir)
    if command == "run":
        fill_template("spark-start-job-template.sh", "start-job.sh", env, job, src_dir, work_dir)
        fill_template("spark-kill-job-template.sh", "kill-job.sh", env, job, src_dir, work_dir)
        fill_template("spark-get-job-status-template.sh", "get-job-status.sh", env, job, src_dir, work_dir)
        fill_template("spark-create-job-output-template.sh", "create-job-output.sh", env, job, src_dir, work_dir)


if __name__ == "__main__":
    main()
