library(jsonlite)
args <- commandArgs(trailingOnly = TRUE)
dir <- args[1]
src.dir <- args[2]

checkAndAssignEnvApiValues <- function(input.api, check.api) {
  if (!is.null(input.api$rSessionPid)) {
    input.api$sessionPid <- input.api$rSessionPid
    input.api$rSessionPid <- NULL
  }
  if (!is.null(input.api$hadoopRPath)) {
    input.api$interpreterPath <- input.api$hadoopRPath
    input.api$hadoopRPath <- NULL
  }
  for (key in names(check.api)) {
    if (key %in% names(input.api)) {
      check.api[key] <- input.api[key]
    } else {
      if (trimws(check.api[key]) == "") {
        stop(paste("DATA PROTOCOL ERROR: Server is missing input data (",key,") from client. Client and server RevoScaleR package version probably does not match."))
      } else {
        warning(paste("DATA PROTOCOL WARNING: Server is missing input data (",key,") from client. Use server side default value. Client and server RevoScaleR package version probably does not match."))
      }
    }
  }

  for (key in names(input.api)) {
    if (key %in% c("envVersion") ) {
      next
    }
    if (!key %in% names(check.api)) {
      warning(paste("DATA PROTOCOL WARNING: Server received unknown input data (",key,") from client. Ignore the data. Client and server RevoScaleR package version probably does not match."))
    }
  }
  check.api$serverType <- "R"
  return(check.api)
}

env <- fromJSON(file.path(dir,"env.json"))
job <- fromJSON(file.path(dir,"job.json"))

if (!"jobType" %in% names(job)) {
  stop("DATA PROTOCOL ERROR: Invalid format of job.json. Missing slot of jobType.")
}
job.type = job["jobType"]

# Currently we don't have plan to migrate hadoop to new workflow
# Just keep it.
if (!job.type %in% c("spark-hpc","spark-hpa","hadoop-hpc","hadoop-hpa")) {
  stop("DATA PROTOCOL ERROR: Invalid value of jobType in job.json. Accepted value: spark-hpc, spark-hpa, hadoop-hpc, hadoop-hpa")
}

if (!"function" %in% names(job)) {
  stop("DATA PROTOCOL ERROR: Missing 'function' value in job.json file. It is required for starting Spark job.")
}

if (!"jobGUID" %in% names(job)) {
  stop("DATA PROTOCOL ERROR: Missing 'jobGUID' value in job.json file. It is required for starting Spark job.")
}

env.api <- fromJSON(file.path(src.dir,"..","templates","env.json"))
env.checked.api <- checkAndAssignEnvApiValues(env, env.api)
env.checked.api <- toJSON(env.checked.api, auto_unbox = TRUE, pretty = TRUE)
write(env.checked.api, file=file.path(dir,"env-final.json"))
